# -*- coding: utf-8 -*-
##############################################################################
#
#    Copyright (C) 2011 Eficent (<http://www.eficent.com/>)
#              Jordi Ballester Alomar <jordi.ballester@eficent.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp import tools
from openerp.osv import fields, osv
from openerp.tools.translate import _

import datetime
import calendar

all_months = (('january', 'January'), ('february', 'February'), ('march', 'March'),
                        ('april', 'April'), ('may', 'May'), ('june', 'June'),
                        ('july', 'July'), ('august', 'August'), ('september', 'September'),
                        ('october', 'October'), ('november', 'November'), ('december', 'December'))

class resource_workload(osv.osv):
    _name = 'resource.workload'
    _description = 'Resource Workload'
    _rec_name = 'month'
    _order = 'sequence'
    
    _columns = {
        'month': fields.selection(all_months, 'Month',),
        'available_hours': fields.integer('Available hours'),
        'effective_hours': fields.integer('Effective Hours'),
        'assigned_hours': fields.integer('Assigned hours'),
        'status': fields.selection((
                        ('red', 'Critical'),
                        ('yellow', 'Warning'),
                        ('green', 'Ok')), 'Status',),
        'resource_id' : fields.many2one('resource.resource', 'Resource', required=True, ondelete='cascade'),
        'sequence': fields.integer('Sequence'),
    }
    
    def compute_control_status(self, cr, uid, ids, context=None):
        if not isinstance(ids, list):
            ids = [ids]
        resource_obj = self.pool.get('resource.resource')
        calendar_obj = self.pool.get('resource.calendar')
        settings_obj = self.pool.get('project.config.settings')
        
        for workload in self.browse(cr, uid, ids, context=context):
            month_number = self.get_month_number(workload.month)
            vals = {}
            resource = workload.resource_id
            resource_calendar = resource.calendar_id
            dt_from = self.get_from_dt(workload.month)
            dt_to = self.get_to_dt(workload.month)
            interval_hours = calendar_obj.interval_hours_get(cr, uid, resource_calendar.id, dt_from, dt_to, resource=resource.id)
            
            effective_hours = interval_hours * resource.time_efficiency
            assigned_hours = self.get_assigned_hours(cr, uid, resource.user_id.id, month_number, context=context)
            
            vals['available_hours'] = interval_hours
            vals['effective_hours'] = effective_hours
            vals['assigned_hours'] = assigned_hours
            vals['sequence'] = (self.get_month_number(workload.month) - datetime.datetime.today().month) % 12
            
            if effective_hours != 0:
                workload_load = 100 * (float(assigned_hours) / float(effective_hours))
            else:
                workload_load = 0
            threashold = settings_obj.get_default_threashold_warning_percentage(cr, uid, [], context=context)
            threashold = threashold.get('threashold_warning_percentage', 100)
            
            vals['status'] = 'green'
            if workload_load > threashold:
                vals['status'] = 'yellow'
            if workload_load > 100:
                vals['status'] = 'red'
            
            self.write(cr, uid, [workload.id], vals, context=context)
    
    def get_assigned_hours(self, cr, uid, user_id, month_number, context=None):
        task_obj = self.pool.get('project.task')
        user_obj = self.pool.get('res.users')
        filters = [('user_id','=',user_id),('state','in',['draft','pending','open'])]
        task_ids = task_obj.search(cr, uid, filters, context=context)
        date_format = '%Y-%m-%d'
        
        assigned_hours = 0
        for task in task_obj.browse(cr, uid, task_ids, context=context):
            if task.date_early_start:
                date_early_start = datetime.datetime.strptime(task.date_early_start, date_format)
                if date_early_start.month == month_number:
                    assigned_hours += task.remaining_hours
        return assigned_hours
    
    def get_from_dt(self, month_name):
        month_number = self.get_month_number(month_name)
        return self.first_day_of_month_rollover(month_number)

    def get_to_dt(self, month_name):
        month_number = self.get_month_number(month_name)
        fist_day = self.first_day_of_month_rollover(month_number)
        return self.last_day_of_month(fist_day)
        
    def get_month_number(self, month_name):
        idx = 0
        for month in all_months:
            idx += 1
            if month[0] == month_name:
                break
        return idx
        
    def first_day_of_month_rollover(self, month_number):
        today = datetime.datetime.today()
        if month_number < today.month:
            return datetime.datetime(today.year+1, month_number, 1, 0, 0, 0)
        else:
            return datetime.datetime(today.year, month_number, 1, 0, 0, 0)

    def last_day_of_month(self, dt):
        if dt.month == 12:
            year = dt.year + 1
        else:
            year = dt.year
        month = dt.month % 12 + 1
        day = 1
        nextMonth = datetime.datetime(year, month, day, 0, 0, 0)
        delta = datetime.timedelta(seconds=1)
        last_day_month = (nextMonth - delta).date()
        return datetime.datetime(last_day_month.year, last_day_month.month, last_day_month.day, 0, 0, 0)

resource_workload()

class resource_resource(osv.osv):
    _inherit = 'resource.resource'
    
    _columns = {
        'workload_ids' : fields.one2many('resource.workload', 'resource_id', 'Monthly Workload'),
        'resource_load': fields.selection((
                        ('red', 'Critical'),
                        ('yellow', 'Warning'),
                        ('green', 'Ok')), 'Resource Load',),
    }

    def all_resources_compute_control_status(self, cr, uid, ids=False, context=None):
        resourse_ids = self.search(cr, uid, [('active','=',True)], context=context)
        self.compute_control_status(cr, 1, resourse_ids, context=context)

    def compute_control_status(self, cr, uid, ids, context=None):
        if not isinstance(ids, list):
            ids = [ids]
        workload_obj = self.pool.get('resource.workload')
        user_obj = self.pool.get('res.users')
        
        for resource in self.browse(cr, uid, ids, context=context):
            if not resource.workload_ids:
                 self.create_initial_workload(cr, uid, [resource.id], context=context)
        
        for resource in self.browse(cr, uid, ids, context=context):
            for workload in resource.workload_ids:
                workload_obj.compute_control_status(cr, uid, [workload.id], context=context)
        
        for resource in self.browse(cr, uid, ids, context=context):
            resource_load = 'green'
            for workload in resource.workload_ids:
                if workload.status == 'yellow':
                    resource_load = 'yellow'
                if workload.status == 'red':
                    resource_load = 'red'
                    break
            self.write(cr, uid, [resource.id], {'resource_load': resource_load}, context=context)
            user_obj.write(cr, uid, [resource.user_id.id], {'resource_load': resource_load}, context=context)
        
    def create_initial_workload(self, cr, uid, ids, context=None):
        print '########################################## workload'
        workload_obj = self.pool.get('resource.workload')
        
        for resource in self.browse(cr, uid, ids, context=context):
            if not resource.workload_ids:
                val = {}
                val['status'] = 'green'
                val['resource_id'] = resource.id
                for month in all_months:
                    val['month'] = month[0]
                    workload_obj.create(cr, uid, val, context=context)
    
    def create(self, cr, uid, vals, context=None):
        super_ret = super(resource_resource, self).create(cr, uid, vals, context=context)
        ret = super_ret
        if ret:
            if not isinstance(ret, list):
                ret = [ret]
            self.create_initial_workload(cr, uid, ret, context=context)
        return super_ret
    
    def write(self, cr, uid, ids, vals, context=None):
        if not isinstance(ids, list):
            ids = [ids]
        ret = super(resource_resource, self).write(cr, uid, ids, vals, context=context)
        
        for resource in self.browse(cr, uid, ids, context=context):
            if not resource.workload_ids:
                 self.create_initial_workload(cr, uid, [resource.id], context=context)
        
        return ret
    
resource_resource()







